-- 古镇秘藏 window
checkRequire("Lua_TownTreasureCellItem");
Lua_TownTreasureWindow = class("Lua_TownTreasureWindow", Lua_Window);

function Lua_TownTreasureWindow:ctor()
	self:superCall("ctor");

	self.isFullSizeWin = true;
	self.showBlackMask = true;
	self.showBackBtn = true;
	self.showWinBg = true;
	self.winBgPath = "/images/common/ui_common_bg_001.jpg";

	self.windowName = "Lua_TownTreasureWindow";
	self.skinName = "TownTreasureWindow_skin";
end

function Lua_TownTreasureWindow:buildPanel(go)
	self:superCall("buildPanel", go);

	self:addLuaEvent("TOWN_TREASURE_INFO_UPDATE", self, self.initWindow);
end

function Lua_TownTreasureWindow:buildShowItem()
	self:superCall("buildShowItem");

	self.isMoving = false;

	self.roleInfoPanel = Lua_RoleInfoPanel.new(self.content);
	self.roleInfoPanel:setRectAnchor(1);

	self:initWindow();
end

function Lua_TownTreasureWindow:dispose()
	if self.roleInfoPanel ~= nil then
		self.roleInfoPanel:dispose();
		self.roleInfoPanel = nil;
	end
	self:superCall("dispose");
end

function Lua_TownTreasureWindow:initWindow()
	self.curVersion = 1;
	self.curCellId = 401001;
	self.cellDic = {};
	local config = getConfig("tower_secrete_cell");
	local vo;
	for k, v in pairs(config) do
		if v.version_id == self.curVersion then
			vo = Lua_TownTreasureCellVo.new(v.id, v);
			self.cellDic[v.id] = vo;
		end
	end

	self.roadArr = Array();
	local startId = DataCenter.townTreasureUtil:getVersionInfo(self.curVersion, "start_id");
	local curId = 0;
	vo = self.cellDic[startId];
	while vo ~= nil and startId ~= curId do
		self.roadArr:add(vo.id);
		curId = vo.next_stage[1];
		vo = self.cellDic[curId];
	end

	self:createMap();

	if self.modelEffImg == nil then
		self.modelEffImg = UICreate.createModelEffImg(self.sub, "/model/role/109000100_low", "109000100_low", 0, 0, 100, 100);
		self.modelEffImg:setCamField(1, 1, 100, true);
		self.modelEffImg:setCamPos(0, 1, -10);
		self.modelEffImg.canRot = false;
		self.modelEffImg.anchorValue = 0;
		self.modelEffImg.pivotValue = 7;
		self.modelEffImg:setAction(0);
		self.modelEffImg:setModelEffDir(120);
	end
	local data = self.cellDic[self.curCellId]
	self:setModelPos(data.posX, -data.posY);
	self:moveModelTo(1200801);
end

function Lua_TownTreasureWindow:createMap()
	self.subRectW, self.subRectH = DataCenter.townTreasureUtil:getMapSize(self.curVersion);
	if self.scrollContainer == nil then
		self.scrollContainer = UICreate.createScrollContainer(self.content, 0, 0, self.width, self.height);
		self.scrollContainer.bCreateContent = true;
		self.scrollContainer.horizontal = true;
		self.scrollContainer.vertical = true;
		self.scrollContainer.movementTypeValue = 2;
		self.scrollContainer.elasticity = 0.3;
		self.scrollContainer.inertia = true;
		self.scrollContainer:setScrollSize(self.subRectW, self.subRectH);
		self.scrollContainer:setScrollPivot(0);
		self.scrollContainer:setScrollAnchor(0, 1, 0, 1);
		-- self.scrollContainer:addRectMask2D();
		self.scrollContainer.siblingIndex = 1;
		self.scrollContainer:addMousDownFuntion(self, self.onMouseDownFun);
	    self.scrollContainer:addMouseUpFuntion(self, self.onMouseUpFun);

		self.sub = self.scrollContainer.scrollContent;
	end

	if self.mapBg == nil then
		self.mapBg = UICreate.createImage(self.sub, "", 0, 0, 0, 0);
		self.mapBg.anchorValue = 0;
		self.mapBg.pivotValue = 0;
	end
	local mapPath = DataCenter.townTreasureUtil:getMapBg(self.curVersion);
	self.mapBg:setUrl(nil);
	self.mapBg:setUrl(mapPath);
	self.mapBg:SetDimensions(self.subRectW, self.subRectH);

	self.cellItemList = {};
	local item;
	for k, v in pairs(self.cellDic) do
		item = self.cellItemList[k];
		if item == nil then
			item = Lua_TownTreasureCellItem.new(self.sub);
			self.cellItemList[k] = item;
		end
		item:setData(v);
	end
end

function Lua_TownTreasureWindow:onMouseDownFun()
	self.startMouseX = CommonUtil.mousePositionX;
	self.startMouseY = CommonUtil.mousePositionY;
	self.startClickTime = getTime();
	self.checkItem = self:checkPressItem();
end

function Lua_TownTreasureWindow:onMouseUpFun()
	if self.checkItem == nil then return end

	local dx = CommonUtil.mousePositionX - self.startMouseX;
	local dy = CommonUtil.mousePositionY - self.startMouseY;
	if getTime() - self.startClickTime <= 0.3 and Mathf.Abs(dx) + Mathf.Abs(dy) < 4 then
		self.checkItem:onCellClick();
	end
end

function Lua_TownTreasureWindow:checkPressItem()
	if self.cellItemList ~= nil then
		local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
		for k, v in pairs(self.cellItemList) do
			if v.isActive and v:checkInBound(winPos.x, winPos.y) then
				return v;
			end
		end
	end
	return nil;
end

function Lua_TownTreasureWindow:setModelPos(posX, posY)
	self.modelPosX = posX;
	self.modelPosY = posY;
	self.modelEffImg:setLocation(posX, posY);
end

function Lua_TownTreasureWindow:moveModelTo(cellId)
	local idx = self.roadArr:indexOf(cellId);
	if idx == -1 or self.isMoving then return end

	self.isMoving = true;
	self.curIdx = self.roadArr:indexOf(self.curCellId);
	self.targetIdx = idx;
	self.modelEffImg:setAction(101);
	self:addTimer("onModelMoving", 0.02, self, self.onModelMoving);
end

function Lua_TownTreasureWindow:onModelMoving()
	if self.curIdx == self.targetIdx then
		self:removeTimer(self, self.onModelMoving);
		self.isMoving = false;
		self.modelEffImg:setAction(0);
		self.modelEffImg:setModelEffDir(120);
		return;
	end

	local idx = self.curIdx + 1;
	if idx > self.roadArr:getSize() then
		idx = 1;
	end
	local data = self.cellDic[self.roadArr[idx]];
	local disX = data.posX - self.modelPosX;
	local disY = -data.posY - self.modelPosY;
	local distance = Mathf.Sqrt(disX * disX + disY * disY);
	if distance <= 2 then
		self.curIdx = idx;
		self:setModelPos(data.posX, -data.posY);
	else
		self:setModelPos(self.modelPosX + disX * 0.1, self.modelPosY + disY * 0.1);
	end

	local angle = math.atan2(-disY, disX) * 180 / 3.1415926 + 90;
	self.modelEffImg:setModelEffDir(angle);
end